// ignore_for_file: non_constant_identifier_names
part of 'google_map_view.widget.dart';

/// 地图控制类
class GoogleMapController extends _Holder
    with
        WidgetsBindingObserver,
        _Community,
        _Pro,
        com_google_android_gms_maps_OnMapReadyCallback {
  /// Android构造器
  GoogleMapController.android(
    com_google_android_gms_maps_MapView _androidController,
    _GoogleMapViewState state,
  ) {
    WidgetsBinding.instance.addObserver(this);
    this.androidController = _androidController;
    this.state = state;

    _androidController.getMapAsync(this);
  }

  /// iOS构造器
  GoogleMapController.ios(
    GMSMapView _iosController,
    _GoogleMapViewState state,
  ) {
    WidgetsBinding.instance.addObserver(this);
    this.iosController = _iosController;
    this.state = state;
  }

  @override
  @mustCallSuper
  Future<void> onMapReady(com_google_android_gms_maps_GoogleMap var1) async {
    await super.onMapReady(var1);
    _androidMap = var1;
  }

  @override
  Future<void> dispose() async {
    await _locateSubscription?.cancel();

    await androidController?.onPause();
    await androidController?.onDestroy();

    WidgetsBinding.instance.removeObserver(this);
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    super.didChangeAppLifecycleState(state);
    debugPrint('didChangeAppLifecycleState: $state');
    // 因为这里的生命周期其实已经是App的生命周期了, 所以除了这里还需要在dispose里释放资源
    switch (state) {
      case AppLifecycleState.resumed:
        androidController?.onResume();
        break;
      case AppLifecycleState.inactive:
        break;
      case AppLifecycleState.paused:
        androidController?.onPause();
        break;
      case AppLifecycleState.detached:
        androidController?.onDestroy();
        break;
    }
  }
}

abstract class _Holder extends java_lang_Object implements IMapController {
  com_google_android_gms_maps_MapView androidController;
  GMSMapView iosController;

  _GoogleMapViewState state;

  // iOS端的回调处理类
  final iosMapDelegate = _IOSMapDelegate();

  // Android端的回调处理类
  final androidMapDelegate = _AndroidMapDelegate();

  // 定位间隔订阅事件
  StreamSubscription _locateSubscription;

  // Android端地图示例
  com_google_android_gms_maps_GoogleMap _androidMap;

  Future<com_google_android_gms_maps_GoogleMap> get androidMap async {
    if (_androidMap != null) {
      return _androidMap;
    } else {
      // 如果地图示例为空, 则等待100毫秒再次递归获取
      await Future.delayed(Duration(milliseconds: 50));
      // ignore: recursive_getters
      return androidMap;
    }
  }
}
