// ignore_for_file: non_constant_identifier_names
part of 'google_map_view.widget.dart';

mixin _Pro on _Holder, _Community {
  @override
  Future<List<IPolygon>> addDistrictOutline(
    String districtName, {
    double width = 5,
    Color strokeColor = Colors.green,
    Color fillColor = Colors.transparent,
    bool onlyMainDistrict = true,
    double sampleRate = 1.0,
  }) {
    throw UnimplementedError();
  }

  @override
  Future<void> addDriveRoute({
    @required LatLng from,
    @required LatLng to,
    List<LatLng> passbyPointList,
    TrafficOption trafficOption,
    double lineWidth = 10,
    ImageProvider customTexture,
  }) {
    throw UnimplementedError();
  }

  @override
  Future<IGroundOverlay> addGroundOverlay(GroundOverlayOption option) {
    throw UnimplementedError();
  }

  @override
  Future<IHeatmapOverlay> addHeatmapTileOverlay(HeatmapTileOption option) {
    throw UnimplementedError();
  }

  @override
  Future<IMultiPointOverlay> addMultiPointOverlay(MultiPointOption option) {
    throw UnimplementedError();
  }

  @override
  Future<IPlaybackTrace> addPlaybackTrace(
    List<LatLng> coordinateList, {
    double width = 5,
    Color strokeColor = Colors.green,
    ImageProvider<Object> iconProvider,
    Duration duration,
  }) {
    throw UnimplementedError();
  }

  @override
  Future<ISmoothMoveMarker> addSmoothMoveMarker(SmoothMoveMarkerOption option) {
    throw UnimplementedError();
  }

  @override
  Future<IUrlTileOverlay> addUrlTileOverlay(UrlTileOption option) {
    throw UnimplementedError();
  }

  @override
  Future<void> clearMarkers(List<IMarker> markers) async {
    await platform(
      android: (pool) async {
        final markerBatch =
            markers.cast<Marker>().map((it) => it.androidModel).toList();
        await markerBatch.remove_batch();
      },
      ios: (pool) async {
        final markerBatch =
            markers.cast<Marker>().map((it) => it.iosModel).toList();
        await markerBatch.set_map_batch(List.filled(markers.length, null));
      },
    );
  }

  /// 截图
  @override
  Future<Uint8List> screenShot() async {
    return platform(
      android: (pool) async {
        final completer = Completer<Uint8List>();

        final map = (await androidMap);
        await map
            .snapshot__com_google_android_gms_maps_GoogleMap_SnapshotReadyCallback(
          androidMapDelegate
            .._onSnapshot = (image) async {
              completer.complete(await image.data);
            },
        );

        return completer.future;
      },
      ios: (pool) async {
        final completer = Completer<Uint8List>();
        // ios端未提供接口, 需要使用系统接口来实现
        // if (@available(iOS 10.0, *)) {
        //       if (_mapView != nil) {
        //         UIGraphicsImageRendererFormat* format = [UIGraphicsImageRendererFormat defaultFormat];
        //         format.scale = [[UIScreen mainScreen] scale];
        //         UIGraphicsImageRenderer* renderer =
        //             [[UIGraphicsImageRenderer alloc] initWithSize:_mapView.frame.size format:format];
        //
        //         UIImage* image = [renderer imageWithActions:^(UIGraphicsImageRendererContext* context) {
        //           [_mapView.layer renderInContext:context.CGContext];
        //         }];
        //         result([FlutterStandardTypedData typedDataWithBytes:UIImagePNGRepresentation(image)]);
        //       } else {
        //         result([FlutterError errorWithCode:@"GoogleMap uninitialized"
        //                                    message:@"takeSnapshot called prior to map initialization"
        //                                    details:nil]);
        //       }
        //     } else {
        //       NSLog(@"Taking snapshots is not supported for Flutter Google Maps prior to iOS 10.");
        //       result(nil);
        //     }
        //
        // await iosController.takeSnapshotInRect_withCompletionBlock(
        //   rect,
        //   (image, state) async {
        //     completer.complete(await image.data);
        //     pool.add(image);
        //   },
        // );
        //
        // pool.add(rect);
        return completer.future;
      },
    );
  }

  @override
  Future<void> setBearing(double bearing, {bool animated = true}) async {
    assert(bearing != null);
    await platform(
      android: (pool) async {
        final map = (await androidMap);

        final position = await map.getCameraPosition();

        final cameraBuilder =
            await com_google_android_gms_maps_model_CameraPosition_Builder
                .create__();
        await cameraBuilder.target(await position.get_target());
        await cameraBuilder.bearing(bearing);

        final update = await com_google_android_gms_maps_CameraUpdateFactory
            .newCameraPosition(await cameraBuilder.build());
        if (animated) {
          await map
              .animateCamera__com_google_android_gms_maps_CameraUpdate(update);
        } else {
          await map.moveCamera(update);
        }

        pool..add(update);
      },
      ios: (pool) async {
        final camera = await iosController.get_camera();

        final target = await camera.get_target();
        final targetCamera = await GMSCameraPosition
            .cameraWithLatitude_longitude_zoom_bearing_viewingAngle(
          await target.latitude,
          await target.longitude,
          await camera.get_zoom(),
          bearing,
          await camera.get_viewingAngle(),
        );
        await iosController.animateToCameraPosition(targetCamera);
      },
    );
  }

  @override
  Future<void> setCameraPosition({
    LatLng coordinate,
    double zoom,
    double tilt,
    double bearing,
    bool animated = true,
    Duration duration = const Duration(milliseconds: 500),
  }) async {
    // TODO duration暂时入口设置
    await platform(
      android: (pool) async {
        final map = (await androidMap);

        final cameraBuilder =
            await com_google_android_gms_maps_model_CameraPosition_Builder
                .create__();

        if (await coordinate.toAndroidModel() != null) {
          await cameraBuilder.target(await coordinate.toAndroidModel());
        }
        if (zoom != null) await cameraBuilder.zoom(tilt);
        if (tilt != null) await cameraBuilder.tilt(tilt);
        if (bearing != null) await cameraBuilder.bearing(tilt);

        final update = await com_google_android_gms_maps_CameraUpdateFactory
            .newCameraPosition(await cameraBuilder.build());
        if (animated) {
          await map
              .animateCamera__com_google_android_gms_maps_CameraUpdate(update);
        } else {
          await map.moveCamera(update);
        }

        pool..add(update);
      },
      ios: (pool) async {
        final targetCamera = await GMSCameraPosition
            .cameraWithLatitude_longitude_zoom_bearing_viewingAngle(
          coordinate.latitude,
          coordinate.longitude,
          zoom,
          bearing,
          tilt,
        );
        await iosController.animateToCameraPosition(targetCamera);
      },
    );
  }

  /// 自定义地图
  @override
  Future<void> setCustomMapStyle({
    @required String styleDataPath,
    @deprecated String styleExtraPath,
    @deprecated String texturePath,
  }) async {
    assert(styleDataPath != null);
    String styleData = await rootBundle.loadString(styleDataPath);

    await platform(
      android: (pool) async {
        final map = (await androidMap);
        // 构造选项
        final option = await com_google_android_gms_maps_model_MapStyleOptions
            .create__String(styleData);
        await map.setMapStyle(option);

        pool..add(option);
      },
      ios: (pool) async {
        // 构造选项
        final option = await GMSMapStyle.styleWithJSONString_error(
          styleData,
          null,
        );

        await iosController.set_mapStyle(option);

        pool.add(option);
      },
    );
  }

  @override
  Future<void> setInfoWindowClickListener(onInfoWindowClicked) {
    throw UnimplementedError();
  }

  @override
  Future<void> setMapAnchor(double anchorU, double anchorV) {
    throw UnimplementedError();
  }

  @override
  Future<void> setMapClickedListener(onMapClick) {
    throw UnimplementedError();
  }

  @override
  Future<void> setMapLongPressedListener(onMapLongPress) {
    throw UnimplementedError();
  }

  @override
  Future<void> setMapMoveListener({onMapMoveStart, onMapMoving, onMapMoveEnd}) {
    throw UnimplementedError();
  }

  @override
  Future<void> setMapRegionLimits(LatLng southWest, LatLng northEast) {
    throw UnimplementedError();
  }

  @override
  Future<void> setMarkerDragListener({
    onMarkerDragStart,
    onMarkerDragging,
    onMarkerDragEnd,
  }) {
    throw UnimplementedError();
  }

  @override
  Future<void> setMaxZoomLevel(double zoomLevel) async {
    assert(zoomLevel != null);
    await platform(
      android: (pool) async {
        final map = (await androidMap);
        await map.setMaxZoomPreference(zoomLevel);
      },
      ios: (pool) async {
        await iosController.setMinZoom_maxZoom(
          await iosController.get_minZoom(),
          zoomLevel,
        );
      },
    );
  }

  @override
  Future<void> setMinZoomLevel(double zoomLevel) async {
    assert(zoomLevel != null);
    await platform(
      android: (pool) async {
        final map = (await androidMap);
        await map.setMinZoomPreference(zoomLevel);
      },
      ios: (pool) async {
        await iosController.setMinZoom_maxZoom(
          zoomLevel,
          await iosController.get_maxZoom(),
        );
      },
    );
  }

  @override
  Future<void> setMyLocationRotateAngle(double angle) {
    throw UnimplementedError();
  }

  @override
  Future<void> setTilt(double tilt, {bool animated = true}) async {
    assert(tilt != null);
    await platform(
      android: (pool) async {
        final map = (await androidMap);

        final position = await map.getCameraPosition();

        final cameraBuilder =
            await com_google_android_gms_maps_model_CameraPosition_Builder
                .create__();
        await cameraBuilder.target(await position.get_target());
        await cameraBuilder.tilt(tilt);
        await cameraBuilder.zoom(await position.get_zoom());
        await cameraBuilder.bearing(await position.get_bearing());

        final update = await com_google_android_gms_maps_CameraUpdateFactory
            .newCameraPosition(await cameraBuilder.build());
        if (animated) {
          await map
              .animateCamera__com_google_android_gms_maps_CameraUpdate(update);
        } else {
          await map.moveCamera(update);
        }

        pool..add(update);
      },
      ios: (pool) async {
        final camera = await iosController.get_camera();

        final target = await camera.get_target();
        final targetCamera = await GMSCameraPosition
            .cameraWithLatitude_longitude_zoom_bearing_viewingAngle(
          await target.latitude,
          await target.longitude,
          await camera.get_zoom(),
          await camera.get_bearing(),
          tilt,
        );
        await iosController.animateToCameraPosition(targetCamera);
      },
    );
  }

  @override
  Future<void> setZoomByCenter(bool byCenter) {
    throw UnimplementedError();
  }

  @override
  Future<void> showBuildings(bool show) {
    throw UnimplementedError();
  }

  @override
  Future<void> showMapText(bool show) {
    throw UnimplementedError();
  }

  @override
  Future<void> showMyLocation(MyLocationOption option) {
    throw UnimplementedError();
  }

  @override
  Future<void> showScaleControl(bool enable) {
    throw UnimplementedError();
  }

  @override
  Future<void> zoomToSpan(
    List<LatLng> bounds, {
    EdgeInsets padding = const EdgeInsets.all(50),
    bool animated = true,
  }) async {
    final double minLat = await Stream.fromIterable(bounds)
        .reduce((pre, cur) => pre.latitude < cur.latitude ? pre : cur)
        .then((bottom) => bottom.latitude);
    final double minLng = await Stream.fromIterable(bounds)
        .reduce((pre, cur) => pre.longitude < cur.longitude ? pre : cur)
        .then((left) => left.longitude);
    final double maxLat = await Stream.fromIterable(bounds)
        .reduce((pre, cur) => pre.latitude > cur.latitude ? pre : cur)
        .then((top) => top.latitude);
    final double maxLng = await Stream.fromIterable(bounds)
        .reduce((pre, cur) => pre.longitude > cur.longitude ? pre : cur)
        .then((right) => right.longitude);
    final devicePixelRatio = MediaQuery.of(state.context).devicePixelRatio;

    await platform(
      android: (pool) async {
        final map = await androidMap;

        // 西南角
        final southWest = await com_google_android_gms_maps_model_LatLng
            .create__double__double(minLat, minLng);
        // 东北角
        final northEast = await com_google_android_gms_maps_model_LatLng
            .create__double__double(maxLat, maxLng);

        // 可视区域矩形
        final rect = await com_google_android_gms_maps_model_LatLngBounds
            .create__com_google_android_gms_maps_model_LatLng__com_google_android_gms_maps_model_LatLng(
                southWest, northEast);

        // 更新对象
        final maxPaddingEdge = [
          padding.left,
          padding.right,
          padding.top,
          padding.bottom
        ].reduce(max);
        final cameraUpdate = await com_google_android_gms_maps_CameraUpdateFactory
            .newLatLngBounds__com_google_android_gms_maps_model_LatLngBounds__int(
          rect,
          (maxPaddingEdge * devicePixelRatio).toInt(),
        );

        if (animated) {
          await map
              .animateCamera__com_google_android_gms_maps_CameraUpdate__int__com_google_android_gms_maps_GoogleMap_CancelableCallback(
            cameraUpdate,
            500,
            null,
          );
        } else {
          await map.moveCamera(cameraUpdate);
        }

        pool..add(southWest)..add(northEast)..add(rect)..add(cameraUpdate);
      },
      ios: (pool) async {
        // 由于屏幕坐标的(0, 0)左上角, 所以需要西北角和东南角
        // 东北角
        final northEast = await CLLocationCoordinate2D.create(maxLat, maxLng);
        // 西南角
        final southWest = await CLLocationCoordinate2D.create(minLat, minLng);

        final rect = await GMSCoordinateBounds.create__();
        await rect.initWithCoordinate_coordinate(northEast, southWest);

        final update = await GMSCameraUpdate.fitBounds_withEdgeInsets(
          rect,
          await UIEdgeInsets.create(
            padding.top,
            padding.left,
            padding.bottom,
            padding.right,
          ),
        );
        if (animated) {
          await iosController.animateWithCameraUpdate(update);
        } else {
          await iosController.moveCamera(update);
        }

        pool..add(northEast)..add(southWest)..add(rect);
      },
    );
  }

  @override
  Future<void> setPadding(EdgeInsets padding) async {
    return platform(
      android: (pool) async {
        final map = await androidMap;
        await map.setPadding(
          (padding.left.toInt() * window.devicePixelRatio).toInt(),
          (padding.right.toInt() * window.devicePixelRatio).toInt(),
          (padding.top.toInt() * window.devicePixelRatio).toInt(),
          (padding.bottom.toInt() * window.devicePixelRatio).toInt(),
        );
      },
      ios: (pool) async {
        await iosController.set_padding(
          await UIEdgeInsets.create(
            padding.top,
            padding.left,
            padding.bottom,
            padding.right,
          ),
        );
      },
    );
  }

  @override
  Future<void> setFps(int fps) async {
    debugPrint('GoogleMap暂未实现');
  }
}
