import 'package:core_location_fluttify/core_location_fluttify.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:google_map_fluttify/src/android/android.export.g.dart';
import 'package:google_map_fluttify/src/android/com/google/android/gms/maps/model/Marker.g.dart';
import 'package:google_map_fluttify/src/ios/GMSMarker.g.dart';
import 'package:google_map_fluttify/src/ios/ios.export.g.dart';
import 'package:uni_map_platform_interface/uni_map_platform_interface.dart';

/// 地图标记
class Marker implements IMarker {
  Marker.android(this.androidModel);

  Marker.ios(this.iosModel);

  com_google_android_gms_maps_model_Marker androidModel;
  GMSMarker iosModel;

  /// 获取标题
  @override
  Future<String> get title {
    return platform(
      android: (_) => androidModel.getTitle(),
      ios: (_) => iosModel.get_title(),
    );
  }

  /// 获取副标题
  @override
  Future<String> get snippet {
    return platform(
      android: (_) => androidModel.getSnippet(),
      ios: (_) => iosModel.get_snippet(),
    );
  }

  /// 获取定位信息
  Future<LatLng> get location {
    return platform(
      android: (_) async {
        final _location = await androidModel.getPosition();
        return LatLng(
          await _location.get_latitude(),
          await _location.get_longitude(),
        );
      },
      ios: (_) async {
        final location = await iosModel.get_position();
        return LatLng(await location.latitude, await location.longitude);
      },
    );
  }

  /// 获取自定义信息
  @override
  Future<String> get object {
    return platform(
      android: (_) {
        return androidModel.getTag().then((object) => object as String);
      },
      ios: (_) {
        return iosModel.get_userData().then((value) => value as String);
      },
    );
  }

  /// 删除marker
  @override
  Future<void> remove() async {
    return platform(
      android: (_) => androidModel.remove(),
      ios: (_) => iosModel.set_map(null),
    );
  }

  /// 设置坐标
  @override
  Future<void> setCoordinate(LatLng coordinate) async {
    assert(coordinate != null);
    return platform(
      android: (_) async => androidModel.setPosition(
        await com_google_android_gms_maps_model_LatLng.create__double__double(
          coordinate.latitude,
          coordinate.longitude,
        ),
      ),
      ios: (_) async => iosModel.set_position(
        await CLLocationCoordinate2D.create(
          coordinate.latitude,
          coordinate.longitude,
        ),
      ),
    );
  }

  /// 设置可见性
  @override
  Future<void> setVisible(bool visible) async {
    assert(visible != null);
    return platform(
      android: (_) => androidModel.setVisible(visible),
      ios: (_) => iosModel.set_opacity(visible ? 1 : 0),
    );
  }

  // /// 显示弹窗
  // Future<void> showInfoWindow() async {
  //   return platform(
  //     android: (_) => androidModel.showInfoWindow(),
  //     ios: (_) => iosModel.show(iosModel, true),
  //   );
  // }

  // /// 关闭弹窗
  // Future<void> hideInfoWindow() async {
  //   return platform(
  //     android: (_) => androidModel.hideInfoWindow(),
  //     ios: (_) => iosController?.deselectAnnotation_animated(iosModel, true),
  //   );
  // }

  /// 设置图标
  @override
  Future<void> setIcon(
    ImageProvider iconProvider,
    ImageConfiguration configuration,
  ) async {
    final iconData = await iconProvider.toImageData(configuration);
    return platform(
      android: (pool) async {
        final bitmap = await android_graphics_Bitmap.create(iconData);
        final icon =
            await com_google_android_gms_maps_model_BitmapDescriptorFactory
                .fromBitmap(bitmap);
        await androidModel.setIcon(icon);
      },
      ios: (pool) async {
        final icon = await UIImage.create(iconData);
        await iosModel.set_icon(icon);
      },
    );
  }

  @override
  Future<LatLng> get coordinate {
    return platform(
      android: (_) async {
        final _location = await androidModel.getPosition();
        return LatLng(
          await _location.get_latitude(),
          await _location.get_longitude(),
        );
      },
      ios: (_) async {
        final location = await iosModel.get_position();
        return LatLng(await location.latitude, await location.longitude);
      },
    );
  }

  @override
  Future<void> hideInfoWindow() {
    throw UnimplementedError();
  }

  @override
  Future<void> setAngle(double angle) {
    throw UnimplementedError();
  }

  @override
  Future<void> showInfoWindow() {
    throw UnimplementedError();
  }

  @override
  Future<void> startAnimation(MarkerAnimation animation) {
    throw UnimplementedError();
  }

  @override
  Future<void> setSnippet(String snippet) {
    // TODO: implement setSnippet
    throw UnimplementedError();
  }

  @override
  Future<void> setTitle(String title) {
    // TODO: implement setTitle
    throw UnimplementedError();
  }
}

/// 折线
class Polyline implements IPolyline {
  Polyline.android(this.androidModel);

  Polyline.ios(this.iosModel);

  com_google_android_gms_maps_model_Polyline androidModel;
  GMSPolyline iosModel;

  @override
  Future<void> remove() {
    return platform(
      android: (_) => androidModel.remove(),
      ios: (_) => iosModel.set_map(null),
    );
  }

  @override
  Future<void> setCoordinateList(List<LatLng> coordinateList) {
    throw UnimplementedError();
  }
}

/// 多边形
class Polygon implements IPolygon {
  Polygon.android(this.androidModel);

  Polygon.ios(this.iosModel);

  com_google_android_gms_maps_model_Polygon androidModel;
  GMSPolygon iosModel;

  @override
  Future<void> remove() {
    return platform(
      android: (_) => androidModel.remove(),
      ios: (_) => iosModel.set_map(null),
    );
  }

  @override
  Future<bool> contains(LatLng target) {
    throw UnimplementedError();
  }
}

/// 圆形
class Circle implements ICircle {
  Circle.android(this.androidModel);

  Circle.ios(this.iosModel);

  com_google_android_gms_maps_model_Circle androidModel;
  GMSCircle iosModel;

  Future<void> remove() {
    return platform(
      android: (_) => androidModel.remove(),
      ios: (_) => iosModel.set_map(null),
    );
  }
}
